@@ -13,6 +13,7 @@ import logging
 import os
 import stat
 import threading
+from typing import Optional, List
 
 from .exception import (CorruptAccountInfo, MissingAccountData)
 from .upload_url_pool import UrlPoolAccountInfo
@@ -23,6 +24,10 @@ logger = logging.getLogger(__name__)
 
 B2_ACCOUNT_INFO_ENV_VAR = 'B2_ACCOUNT_INFO'
 B2_ACCOUNT_INFO_DEFAULT_FILE = '~/.b2_account_info'
+XDG_CONFIG_HOME_ENV_VAR = 'XDG_CONFIG_HOME'
+
+DEFAULT_ABSOLUTE_MINIMUM_PART_SIZE = 5000000  # this value is used ONLY in migrating db, and in v1 wrapper, it is not
+# meant to be a default for other applications
 
 
 class SqliteAccountInfo(UrlPoolAccountInfo):
@@ -36,22 +41,59 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
 
     def __init__(self, file_name=None, last_upgrade_to_run=None):
         """
-        If ``file_name`` argument is empty or ``None``, path from ``B2_ACCOUNT_INFO`` environment variable is used. If that is not available, a default of ``~/.b2_account_info`` is used.
+        Initialize SqliteAccountInfo.
+
+        The exact algorithm used to determine the location of the database file is not API in any sense.
+        If the location of the database file is required (for cleanup, etc), do not assume a specific resolution:
+        instead, use ``self.filename`` to get the actual resolved location.
+
+        SqliteAccountInfo currently checks locations in the following order:
+
+        * ``file_name``, if truthy
+        * ``{B2_ACCOUNT_INFO_ENV_VAR}`` env var's value, if set
+        * ``{B2_ACCOUNT_INFO_DEFAULT_FILE}``, if it exists
+        * ``{XDG_CONFIG_HOME_ENV_VAR}/b2/account_info``, if ``{XDG_CONFIG_HOME_ENV_VAR}`` env var is set
+        * ``{B2_ACCOUNT_INFO_DEFAULT_FILE}``, as default
+
+        If the directory ``{XDG_CONFIG_HOME_ENV_VAR}/b2`` does not exist (and is needed), it is created.
 
         :param str file_name: The sqlite file to use; overrides the default.
         :param int last_upgrade_to_run: For testing only, override the auto-update on the db.
         """
         self.thread_local = threading.local()
-        user_account_info_path = file_name or os.environ.get(
-            B2_ACCOUNT_INFO_ENV_VAR, B2_ACCOUNT_INFO_DEFAULT_FILE
-        )
-        self.filename = file_name or os.path.expanduser(user_account_info_path)
+
+        if file_name:
+            user_account_info_path = file_name
+        elif B2_ACCOUNT_INFO_ENV_VAR in os.environ:
+            user_account_info_path = os.environ[B2_ACCOUNT_INFO_ENV_VAR]
+        elif os.path.exists(os.path.expanduser(B2_ACCOUNT_INFO_DEFAULT_FILE)):
+            user_account_info_path = B2_ACCOUNT_INFO_DEFAULT_FILE
+        elif XDG_CONFIG_HOME_ENV_VAR in os.environ:
+            config_home = os.environ[XDG_CONFIG_HOME_ENV_VAR]
+            user_account_info_path = os.path.join(config_home, 'b2', 'account_info')
+            if not os.path.exists(os.path.join(config_home, 'b2')):
+                os.makedirs(os.path.join(config_home, 'b2'), mode=0o755)
+        else:
+            user_account_info_path = B2_ACCOUNT_INFO_DEFAULT_FILE
+
+        self.filename = os.path.expanduser(user_account_info_path)
         logger.debug('%s file path to use: %s', self.__class__.__name__, self.filename)
-        self._validate_database()
+
+        self._validate_database(last_upgrade_to_run)
         with self._get_connection() as conn:
             self._create_tables(conn, last_upgrade_to_run)
         super(SqliteAccountInfo, self).__init__()
 
+    # dirty trick to use parameters in the docstring
+    if getattr(__init__, '__doc__', None):  # don't break when using `python -oo`
+        __init__.__doc__ = __init__.__doc__.format(
+            **dict(
+                B2_ACCOUNT_INFO_ENV_VAR=B2_ACCOUNT_INFO_ENV_VAR,
+                B2_ACCOUNT_INFO_DEFAULT_FILE=B2_ACCOUNT_INFO_DEFAULT_FILE,
+                XDG_CONFIG_HOME_ENV_VAR=XDG_CONFIG_HOME_ENV_VAR,
+            )
+        )
+
     def _validate_database(self, last_upgrade_to_run=None):
         """
         Make sure that the database is openable.  Removes the file if it's not.
@@ -89,11 +131,17 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
                     self._create_tables(conn, last_upgrade_to_run)
                     insert_statement = """
                         INSERT INTO account
-                        (account_id, application_key, account_auth_token, api_url, download_url, minimum_part_size, realm)
-                        values (?, ?, ?, ?, ?, ?, ?);
+                        (account_id, application_key, account_auth_token, api_url, download_url, 
+                         recommended_part_size, realm, absolute_minimum_part_size)
+                        values (?, ?, ?, ?, ?, ?, ?, ?);
                     """
-
-                    conn.execute(insert_statement, tuple(data[k] for k in keys))
+                    # Migrating from old schema is a little confusing, but the values change as:
+                    # minimum_part_size -> recommended_part_size
+                    # new column absolute_minimum_part_size = DEFAULT_ABSOLUTE_MINIMUM_PART_SIZE
+                    conn.execute(
+                        insert_statement,
+                        (*(data[k] for k in keys), DEFAULT_ABSOLUTE_MINIMUM_PART_SIZE)
+                    )
                 # all is happy now
                 return
         except ValueError:  # includes json.decoder.JSONDecodeError
@@ -177,18 +225,110 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
         """
         )
         # By default, we run all the upgrades
-        last_upgrade_to_run = 3 if last_upgrade_to_run is None else last_upgrade_to_run
+        last_upgrade_to_run = 4 if last_upgrade_to_run is None else last_upgrade_to_run
         # Add the 'allowed' column if it hasn't been yet.
         if 1 <= last_upgrade_to_run:
-            self._ensure_update(1, 'ALTER TABLE account ADD COLUMN allowed TEXT;')
+            self._ensure_update(1, ['ALTER TABLE account ADD COLUMN allowed TEXT;'])
         # Add the 'account_id_or_app_key_id' column if it hasn't been yet
         if 2 <= last_upgrade_to_run:
-            self._ensure_update(2, 'ALTER TABLE account ADD COLUMN account_id_or_app_key_id TEXT;')
+            self._ensure_update(
+                2, ['ALTER TABLE account ADD COLUMN account_id_or_app_key_id TEXT;']
+            )
         # Add the 's3_api_url' column if it hasn't been yet
         if 3 <= last_upgrade_to_run:
-            self._ensure_update(3, 'ALTER TABLE account ADD COLUMN s3_api_url TEXT;')
+            self._ensure_update(3, ['ALTER TABLE account ADD COLUMN s3_api_url TEXT;'])
+        if 4 <= last_upgrade_to_run:
+            self._ensure_update(
+                4, [
+                    """
+                    CREATE TABLE
+                    tmp_account (
+                        account_id TEXT NOT NULL,
+                        application_key TEXT NOT NULL,
+                        account_auth_token TEXT NOT NULL,
+                        api_url TEXT NOT NULL,
+                        download_url TEXT NOT NULL,
+                        absolute_minimum_part_size INT NOT NULL DEFAULT {},
+                        recommended_part_size INT NOT NULL,
+                        realm TEXT NOT NULL,
+                        allowed TEXT,
+                        account_id_or_app_key_id TEXT,
+                        s3_api_url TEXT    
+                    );
+                    """.format(DEFAULT_ABSOLUTE_MINIMUM_PART_SIZE),
+                    """INSERT INTO tmp_account(
+                        account_id,
+                        application_key,
+                        account_auth_token,
+                        api_url,
+                        download_url,
+                        recommended_part_size,
+                        realm,
+                        allowed,
+                        account_id_or_app_key_id,
+                        s3_api_url
+                    ) 
+                    SELECT
+                        account_id,
+                        application_key,
+                        account_auth_token,
+                        api_url,
+                        download_url,
+                        minimum_part_size,
+                        realm,
+                        allowed,
+                        account_id_or_app_key_id,
+                        s3_api_url
+                    FROM account;
+                    """,
+                    'DROP TABLE account;',
+                    """
+                    CREATE TABLE account (
+                        account_id TEXT NOT NULL,
+                        application_key TEXT NOT NULL,
+                        account_auth_token TEXT NOT NULL,
+                        api_url TEXT NOT NULL,
+                        download_url TEXT NOT NULL,
+                        absolute_minimum_part_size INT NOT NULL,
+                        recommended_part_size INT NOT NULL,
+                        realm TEXT NOT NULL,
+                        allowed TEXT,
+                        account_id_or_app_key_id TEXT,
+                        s3_api_url TEXT    
+                    );
+                    """,
+                    """INSERT INTO account(
+                                    account_id,
+                                    application_key,
+                                    account_auth_token,
+                                    api_url,
+                                    download_url,
+                                    absolute_minimum_part_size,
+                                    recommended_part_size,
+                                    realm,
+                                    allowed,
+                                    account_id_or_app_key_id,
+                                    s3_api_url
+                                ) 
+                                SELECT
+                                    account_id,
+                                    application_key,
+                                    account_auth_token,
+                                    api_url,
+                                    download_url,
+                                    absolute_minimum_part_size,
+                                    recommended_part_size,
+                                    realm,
+                                    allowed,
+                                    account_id_or_app_key_id,
+                                    s3_api_url
+                                FROM tmp_account;
+                                """,
+                    'DROP TABLE tmp_account;',
+                ]
+            )
 
-    def _ensure_update(self, update_number, update_command):
+    def _ensure_update(self, update_number, update_commands: List[str]):
         """
         Run the update with the given number if it hasn't been done yet.
 
@@ -202,9 +342,9 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
                 (update_number,)
             )
             update_count = cursor.fetchone()[0]
-            assert update_count in [0, 1, 2]
             if update_count == 0:
-                conn.execute(update_command)
+                for command in update_commands:
+                    conn.execute(command)
                 conn.execute(
                     'INSERT INTO update_done (update_number) VALUES (?);', (update_number,)
                 )
@@ -219,8 +359,18 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
             conn.execute('DELETE FROM bucket_upload_url;')
 
     def _set_auth_data(
-        self, account_id, auth_token, api_url, download_url, minimum_part_size, application_key,
-        realm, s3_api_url, allowed, application_key_id
+        self,
+        account_id,
+        auth_token,
+        api_url,
+        download_url,
+        recommended_part_size,
+        absolute_minimum_part_size,
+        application_key,
+        realm,
+        s3_api_url,
+        allowed,
+        application_key_id,
     ):
         assert self.allowed_is_valid(allowed)
         with self._get_connection() as conn:
@@ -229,8 +379,9 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
             conn.execute('DELETE FROM bucket_upload_url;')
             insert_statement = """
                 INSERT INTO account
-                (account_id, account_id_or_app_key_id, application_key, account_auth_token, api_url, download_url, minimum_part_size, realm, allowed, s3_api_url)
-                values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?);
+                (account_id, account_id_or_app_key_id, application_key, account_auth_token, api_url, download_url, 
+                 recommended_part_size, absolute_minimum_part_size, realm, allowed, s3_api_url)
+                values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);
             """
 
             conn.execute(
@@ -241,7 +392,8 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
                     auth_token,
                     api_url,
                     download_url,
-                    minimum_part_size,
+                    recommended_part_size,
+                    absolute_minimum_part_size,
                     realm,
                     json.dumps(allowed),
                     s3_api_url,
@@ -327,12 +479,29 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
     def get_realm(self):
         return self._get_account_info_or_raise('realm')
 
-    def get_minimum_part_size(self):
-        return self._get_account_info_or_raise('minimum_part_size')
+    def get_recommended_part_size(self):
+        return self._get_account_info_or_raise('recommended_part_size')
+
+    def get_absolute_minimum_part_size(self):
+        return self._get_account_info_or_raise('absolute_minimum_part_size')
 
     def get_allowed(self):
         """
         Return 'allowed' dictionary info.
+        Example:
+
+        .. code-block:: python
+
+            {
+                "bucketId": null,
+                "bucketName": null,
+                "capabilities": [
+                    "listKeys",
+                    "writeKeys"
+                ],
+                "namePrefix": null
+            }
+
         The 'allowed' column was not in the original schema, so it may be NULL.
 
         :rtype: dict
@@ -385,11 +554,17 @@ class SqliteAccountInfo(UrlPoolAccountInfo):
             conn.execute('DELETE FROM bucket WHERE bucket_name = ?;', (bucket_name,))
 
     def get_bucket_id_or_none_from_bucket_name(self, bucket_name):
+        return self._safe_query(
+            'SELECT bucket_id FROM bucket WHERE bucket_name = ?;', (bucket_name,)
+        )
+
+    def get_bucket_name_or_none_from_bucket_id(self, bucket_id: str) -> Optional[str]:
+        return self._safe_query('SELECT bucket_name FROM bucket WHERE bucket_id = ?;', (bucket_id,))
+
+    def _safe_query(self, query, params):
         try:
             with self._get_connection() as conn:
-                cursor = conn.execute(
-                    'SELECT bucket_id FROM bucket WHERE bucket_name = ?;', (bucket_name,)
-                )
+                cursor = conn.execute(query, params)
                 return cursor.fetchone()[0]
         except TypeError:  # TypeError: 'NoneType' object is unsubscriptable
             return None
