@@ -9,11 +9,11 @@
 ######################################################################
 
 import logging
-from typing import Optional
+from typing import Optional, Tuple
 
 from .encryption.setting import EncryptionSetting, EncryptionSettingFactory
 from .encryption.types import EncryptionMode
-from .exception import FileNotPresent, FileOrBucketNotFound, UnexpectedCloudBehaviour, UnrecognizedBucketType
+from .exception import BucketIdNotFound, FileNotPresent, FileOrBucketNotFound, UnexpectedCloudBehaviour, UnrecognizedBucketType
 from .file_lock import (
     BucketRetentionSetting,
     FileLockConfiguration,
@@ -21,10 +21,11 @@ from .file_lock import (
     UNKNOWN_BUCKET_RETENTION,
     LegalHold,
 )
-from .file_version import FileVersionInfo, FileVersionInfoFactory
-from .progress import DoNothingProgressListener
+from .file_version import DownloadVersion, FileVersion
+from .progress import AbstractProgressListener, DoNothingProgressListener
 from .transfer.emerge.executor import AUTO_CONTENT_TYPE
 from .transfer.emerge.write_intent import WriteIntent
+from .transfer.inbound.downloaded_file import DownloadedFile
 from .transfer.outbound.copy_source import CopySource
 from .transfer.outbound.upload_source import UploadSourceBytes, UploadSourceLocalFile
 from .utils import B2TraceMeta, disable_trace, limit_trace_arguments
@@ -39,7 +40,6 @@ class Bucket(metaclass=B2TraceMeta):
     """
 
     DEFAULT_CONTENT_TYPE = AUTO_CONTENT_TYPE
-    FILE_VERSION_FACTORY = staticmethod(FileVersionInfoFactory)
 
     def __init__(
         self,
@@ -88,6 +88,16 @@ class Bucket(metaclass=B2TraceMeta):
         self.default_retention = default_retention
         self.is_file_lock_enabled = is_file_lock_enabled
 
+    def get_fresh_state(self) -> 'Bucket':
+        """
+        Fetch all the information about this bucket and return a new bucket object.
+        This method does NOT change the object it is called on.
+        """
+        buckets_found = self.api.list_buckets(bucket_id=self.id_)
+        if not buckets_found:
+            raise BucketIdNotFound(self.id_)
+        return buckets_found[0]
+
     def get_id(self):
         """
         Return bucket ID.
@@ -115,37 +125,39 @@ class Bucket(metaclass=B2TraceMeta):
 
     def update(
         self,
-        bucket_type=None,
-        bucket_info=None,
-        cors_rules=None,
-        lifecycle_rules=None,
-        if_revision_is=None,
+        bucket_type: Optional[str] = None,
+        bucket_info: Optional[dict] = None,
+        cors_rules: Optional[dict] = None,
+        lifecycle_rules: Optional[dict] = None,
+        if_revision_is: Optional[int] = None,
         default_server_side_encryption: Optional[EncryptionSetting] = None,
         default_retention: Optional[BucketRetentionSetting] = None,
     ):
         """
         Update various bucket parameters.
-        For legacy reasons in apiver v1 it returns whatever server returned on b2_update_bucket call, v2 will change that.
 
-        :param str bucket_type: a bucket type
-        :param dict bucket_info: an info to store with a bucket
-        :param dict cors_rules: CORS rules to store with a bucket
-        :param dict lifecycle_rules: lifecycle rules to store with a bucket
-        :param int if_revision_is: revision number, update the info **only if** *revision* equals to *if_revision_is*
-        :param b2sdk.v1.EncryptionSetting default_server_side_encryption: default server side encryption settings (``None`` if unknown)
-        :param b2sdk.v1.BucketRetentionSetting default_retention: bucket default retention setting
+        :param bucket_type: a bucket type, e.g. ``allPrivate`` or ``allPublic``
+        :param bucket_info: an info to store with a bucket
+        :param cors_rules: CORS rules to store with a bucket
+        :param lifecycle_rules: lifecycle rules to store with a bucket
+        :param if_revision_is: revision number, update the info **only if** *revision* equals to *if_revision_is*
+        :param default_server_side_encryption: default server side encryption settings (``None`` if unknown)
+        :param default_retention: bucket default retention setting
         """
         account_id = self.api.account_info.get_account_id()
-        return self.api.session.update_bucket(
-            account_id,
-            self.id_,
-            bucket_type=bucket_type,
-            bucket_info=bucket_info,
-            cors_rules=cors_rules,
-            lifecycle_rules=lifecycle_rules,
-            if_revision_is=if_revision_is,
-            default_server_side_encryption=default_server_side_encryption,
-            default_retention=default_retention,
+        return self.api.BUCKET_FACTORY_CLASS.from_api_bucket_dict(
+            self.api,
+            self.api.session.update_bucket(
+                account_id,
+                self.id_,
+                bucket_type=bucket_type,
+                bucket_info=bucket_info,
+                cors_rules=cors_rules,
+                lifecycle_rules=lifecycle_rules,
+                if_revision_is=if_revision_is,
+                default_server_side_encryption=default_server_side_encryption,
+                default_retention=default_retention,
+            )
         )
 
     def cancel_large_file(self, file_id):
@@ -158,32 +170,24 @@ class Bucket(metaclass=B2TraceMeta):
 
     def download_file_by_id(
         self,
-        file_id,
-        download_dest,
-        progress_listener=None,
-        range_=None,
+        file_id: str,
+        progress_listener: Optional[AbstractProgressListener] = None,
+        range_: Optional[Tuple[int, int]] = None,
         encryption: Optional[EncryptionSetting] = None,
-    ):
+    ) -> DownloadedFile:
         """
         Download a file by ID.
 
         .. note::
           download_file_by_id actually belongs in :py:class:`b2sdk.v1.B2Api`, not in :py:class:`b2sdk.v1.Bucket`; we just provide a convenient redirect here
 
-        :param str file_id: a file ID
-        :param download_dest: an instance of the one of the following classes: \
-        :class:`~b2sdk.v1.DownloadDestLocalFile`,\
-        :class:`~b2sdk.v1.DownloadDestBytes`,\
-        :class:`~b2sdk.v1.DownloadDestProgressWrapper`,\
-        :class:`~b2sdk.v1.PreSeekedDownloadDest`,\
-        or any sub class of :class:`~b2sdk.v1.AbstractDownloadDestination`
-        :param b2sdk.v1.AbstractProgressListener, None progress_listener: a progress listener object to use, or ``None`` to not report progress
-        :param tuple[int, int] range_: two integer values, start and end offsets
-        :param b2sdk.v1.EncryptionSetting encryption: encryption settings (``None`` if unknown)
+        :param file_id: a file ID
+        :param progress_listener: a progress listener object to use, or ``None`` to not track progress
+        :param range_: two integer values, start and end offsets
+        :param encryption: encryption settings (``None`` if unknown)
         """
         return self.api.download_file_by_id(
             file_id,
-            download_dest,
             progress_listener,
             range_=range_,
             encryption=encryption,
@@ -191,12 +195,11 @@ class Bucket(metaclass=B2TraceMeta):
 
     def download_file_by_name(
         self,
-        file_name,
-        download_dest,
-        progress_listener=None,
-        range_=None,
+        file_name: str,
+        progress_listener: Optional[AbstractProgressListener] = None,
+        range_: Optional[Tuple[int, int]] = None,
         encryption: Optional[EncryptionSetting] = None,
-    ):
+    ) -> DownloadedFile:
         """
         Download a file by name.
 
@@ -204,44 +207,36 @@ class Bucket(metaclass=B2TraceMeta):
 
             :ref:`Synchronizer <sync>`, a *high-performance* utility that synchronizes a local folder with a Bucket.
 
-        :param str file_name: a file name
-        :param download_dest: an instance of the one of the following classes: \
-        :class:`~b2sdk.v1.DownloadDestLocalFile`,\
-        :class:`~b2sdk.v1.DownloadDestBytes`,\
-        :class:`~b2sdk.v1.DownloadDestProgressWrapper`,\
-        :class:`~b2sdk.v1.PreSeekedDownloadDest`,\
-        or any sub class of :class:`~b2sdk.v1.AbstractDownloadDestination`
-        :param b2sdk.v1.AbstractProgressListener, None progress_listener: a progress listener object to use, or ``None`` to not track progress
-        :param tuple[int, int] range_: two integer values, start and end offsets
-        :param b2sdk.v1.EncryptionSetting encryption: encryption settings (``None`` if unknown)
+        :param file_name: a file name
+        :param progress_listener: a progress listener object to use, or ``None`` to not track progress
+        :param range_: two integer values, start and end offsets
+        :param encryption: encryption settings (``None`` if unknown)
         """
         url = self.api.session.get_download_url_by_name(self.name, file_name)
         return self.api.services.download_manager.download_file_from_url(
             url,
-            download_dest,
             progress_listener,
             range_,
             encryption=encryption,
         )
 
-    def get_file_info_by_id(self, file_id: str) -> FileVersionInfo:
+    def get_file_info_by_id(self, file_id: str) -> FileVersion:
         """
-        Gets a file version's info by ID.
+        Gets a file version's by ID.
 
         :param str file_id: the id of the file who's info will be retrieved.
         :rtype: generator[b2sdk.v1.FileVersionInfo]
         """
-        return self.FILE_VERSION_FACTORY.from_api_response(self.api.get_file_info(file_id))
+        return self.api.get_file_info(file_id)
 
-    def get_file_info_by_name(self, file_name: str) -> FileVersionInfo:
+    def get_file_info_by_name(self, file_name: str) -> DownloadVersion:
         """
-        Gets a file version's info by its name.
+        Gets a file's DownloadVersion by name.
 
         :param str file_name: the name of the file who's info will be retrieved.
-        :rtype: generator[b2sdk.v1.FileVersionInfo]
         """
         try:
-            return self.FILE_VERSION_FACTORY.from_response_headers(
+            return self.api.download_version_factory.from_response_headers(
                 self.api.session.get_file_info_by_name(self.name, file_name)
             )
         except FileOrBucketNotFound:
@@ -290,17 +285,23 @@ class Bucket(metaclass=B2TraceMeta):
             )
 
             for entry in response['files']:
-                file_version_info = self.FILE_VERSION_FACTORY.from_api_response(entry)
-                if file_version_info.file_name != file_name:
+                file_version = self.api.file_version_factory.from_api_response(entry)
+                if file_version.file_name != file_name:
                     # All versions for the requested file name have been listed.
                     return
-                yield file_version_info
+                yield file_version
             start_file_name = response['nextFileName']
             start_file_id = response['nextFileId']
             if start_file_name is None:
                 return
 
-    def ls(self, folder_to_list='', show_versions=False, recursive=False, fetch_count=10000):
+    def ls(
+        self,
+        folder_to_list: str = '',
+        latest_only: bool = True,
+        recursive: bool = False,
+        fetch_count: Optional[int] = 10000
+    ):
         """
         Pretend that folders exist and yields the information about the files in a folder.
 
@@ -312,14 +313,14 @@ class Bucket(metaclass=B2TraceMeta):
         When the `recursive` flag is set, lists all of the files in the given
         folder, and all of its sub-folders.
 
-        :param str folder_to_list: the name of the folder to list; must not start with "/".
+        :param folder_to_list: the name of the folder to list; must not start with "/".
                                Empty string means top-level folder
-        :param bool show_versions: when ``True`` returns info about all versions of a file,
-                              when ``False``, just returns info about the most recent versions
-        :param bool recursive: if ``True``, list folders recursively
-        :param int,None fetch_count: how many entries to return or ``None`` to use the default. Acceptable values: 1 - 10000
+        :param latest_only: when ``False`` returns info about all versions of a file,
+                              when ``True``, just returns info about the most recent versions
+        :param recursive: if ``True``, list folders recursively
+        :param fetch_count: how many entries to return or ``None`` to use the default. Acceptable values: 1 - 10000
         :rtype: generator[tuple[b2sdk.v1.FileVersionInfo, str]]
-        :returns: generator of (file_version_info, folder_name) tuples
+        :returns: generator of (file_version, folder_name) tuples
 
         .. note::
             In case of `recursive=True`, folder_name is returned only for first file in the folder.
@@ -343,22 +344,22 @@ class Bucket(metaclass=B2TraceMeta):
         start_file_id = None
         session = self.api.session
         while True:
-            if show_versions:
+            if latest_only:
+                response = session.list_file_names(self.id_, start_file_name, fetch_count, prefix)
+            else:
                 response = session.list_file_versions(
                     self.id_, start_file_name, start_file_id, fetch_count, prefix
                 )
-            else:
-                response = session.list_file_names(self.id_, start_file_name, fetch_count, prefix)
             for entry in response['files']:
-                file_version_info = self.FILE_VERSION_FACTORY.from_api_response(entry)
-                if not file_version_info.file_name.startswith(prefix):
+                file_version = self.api.file_version_factory.from_api_response(entry)
+                if not file_version.file_name.startswith(prefix):
                     # We're past the files we care about
                     return
-                after_prefix = file_version_info.file_name[len(prefix):]
+                after_prefix = file_version.file_name[len(prefix):]
                 if '/' not in after_prefix or recursive:
                     # This is not a folder, so we'll print it out and
                     # continue on.
-                    yield file_version_info, None
+                    yield file_version, None
                     current_dir = None
                 else:
                     # This is a folder.  If it's different than the folder
@@ -368,7 +369,7 @@ class Bucket(metaclass=B2TraceMeta):
                     folder_with_slash = after_prefix.split('/')[0] + '/'
                     if folder_with_slash != current_dir:
                         folder_name = prefix + folder_with_slash
-                        yield file_version_info, folder_name
+                        yield file_version, folder_name
                         current_dir = folder_with_slash
             if response['nextFileName'] is None:
                 # The response says there are no more files in the bucket,
@@ -788,7 +789,7 @@ class Bucket(metaclass=B2TraceMeta):
         :rtype: b2sdk.v1.FileVersionInfo
         """
         response = self.api.session.hide_file(self.id_, file_name)
-        return self.FILE_VERSION_FACTORY.from_api_response(response)
+        return self.api.file_version_factory.from_api_response(response)
 
     def copy(
         self,
@@ -948,12 +949,12 @@ class BucketFactory(object):
                    }
                },
                "fileLockConfiguration": {
-                   "isClientAuthorizedToRead": true, 
+                   "isClientAuthorizedToRead": true,
                    "value": {
                        "defaultRetention": {
                            "mode": null,
                            "period": null
-                        }, 
+                        },
                         "isFileLockEnabled": false
                     }
               }
