@@ -8,28 +8,41 @@
 #
 ######################################################################
 
+import enum
 import logging
-from typing import Optional
+from typing import Optional, Union
 import urllib
 import urllib.parse
 
+from ..http_constants import SSE_C_KEY_ID_FILE_INFO_KEY_NAME, SSE_C_KEY_ID_HEADER
 from ..utils import b64_of_bytes, md5_of_bytes
 from .types import ENCRYPTION_MODES_WITH_MANDATORY_ALGORITHM, ENCRYPTION_MODES_WITH_MANDATORY_KEY
 from .types import EncryptionAlgorithm, EncryptionMode
 
-SSE_C_KEY_ID_FILE_INFO_KEY_NAME = 'sse_c_key_id'
-
 logger = logging.getLogger(__name__)
 
 
+class _UnknownKeyId(enum.Enum):
+    """The purpose of this enum is to provide a sentinel that can be used with type annotations."""
+    unknown_key_id = 0
+
+
+UNKNOWN_KEY_ID = _UnknownKeyId.unknown_key_id
+"""
+Value for EncryptionKey.key_id that signifies that the key id may or may not be defined. Useful when establishing
+encryption settings based on user input (and not based on B2 cloud data).
+"""
+
+
 class EncryptionKey:
     """
     Hold information about encryption key: the key itself, and its id. The id may be None, if it's not set
-    in encrypted file's fileInfo. The secret may be None, if encryption metadata is read from the server.
+    in encrypted file's fileInfo, or UNKNOWN_KEY_ID when that information is missing.
+    The secret may be None, if encryption metadata is read from the server.
     """
     SECRET_REPR = '******'
 
-    def __init__(self, secret: Optional[bytes], key_id: Optional[str]):
+    def __init__(self, secret: Optional[bytes], key_id: Union[str, None, _UnknownKeyId]):
         self.secret = secret
         self.key_id = key_id
 
@@ -40,7 +53,12 @@ class EncryptionKey:
         key_repr = self.SECRET_REPR
         if self.secret is None:
             key_repr = None
-        return '<%s(%s, %s)>' % (self.__class__.__name__, key_repr, self.key_id)
+
+        if self.key_id is UNKNOWN_KEY_ID:
+            key_id_repr = 'unknown'
+        else:
+            key_id_repr = repr(self.key_id)
+        return '<%s(%s, %s)>' % (self.__class__.__name__, key_repr, key_id_repr)
 
     def as_dict(self):
         """
@@ -145,9 +163,11 @@ class EncryptionSetting:
         elif self.mode == EncryptionMode.SSE_B2:
             self._add_sse_b2_headers(headers)
         elif self.mode == EncryptionMode.SSE_C:
+            if self.key.key_id is UNKNOWN_KEY_ID:
+                raise ValueError('Cannot upload a file with an unknown key id')
             self._add_sse_c_headers(headers)
             if self.key.key_id is not None:
-                header = 'X-Bz-Info-%s' % (SSE_C_KEY_ID_FILE_INFO_KEY_NAME,)
+                header = SSE_C_KEY_ID_HEADER
                 if headers.get(header) is not None and headers[header] != self.key.key_id:
                     raise ValueError(
                         'Ambiguous key id set: "%s" in headers and "%s" in %s' %
@@ -180,6 +200,8 @@ class EncryptionSetting:
     def add_key_id_to_file_info(self, file_info: Optional[dict]):
         if self.key is None or self.key.key_id is None:
             return file_info
+        if self.key.key_id is UNKNOWN_KEY_ID:
+            raise ValueError('Cannot add an unknown key id to file info')
         if file_info is None:
             file_info = {}
         if file_info.get(SSE_C_KEY_ID_FILE_INFO_KEY_NAME) is not None and file_info[
@@ -206,7 +228,7 @@ class EncryptionSettingFactory:
     # if not authorized to read:
     #    isClientAuthorizedToRead is False and there is no value, so no mode
     #
-    # BUT file_version_info (get_file_info, list_file_versions, upload_file etc)
+    # BUT file_version (get_file_info, list_file_versions, upload_file etc)
     # if the file is encrypted, then
     #     "serverSideEncryption": {"algorithm": "AES256", "mode": "SSE-B2"},
     #     or
@@ -309,18 +331,19 @@ class EncryptionSettingFactory:
 
     @classmethod
     def from_response_headers(cls, headers):
-
-        mode = EncryptionMode(headers.get('X-Bz-Server-Side-Encryption', 'none'))
-        kwargs = {
-            'mode': mode,
-        }
-        if mode == EncryptionMode.SSE_C:
-            kwargs['key'] = EncryptionKey(secret=None, key_id=None)
-        algorithm = headers.get('X-Bz-Server-Side-Encryption-Customer-Algorithm')
-        if algorithm is not None:
-            kwargs['algorithm'] = EncryptionAlgorithm(algorithm)
-
-        return EncryptionSetting(**kwargs)
+        if 'X-Bz-Server-Side-Encryption' in headers:
+            mode = EncryptionMode.SSE_B2
+            algorithm = EncryptionAlgorithm(headers['X-Bz-Server-Side-Encryption'])
+            return EncryptionSetting(mode, algorithm)
+        if 'X-Bz-Server-Side-Encryption-Customer-Algorithm' in headers:
+            mode = EncryptionMode.SSE_C
+            algorithm = EncryptionAlgorithm(
+                headers['X-Bz-Server-Side-Encryption-Customer-Algorithm']
+            )
+            key_id = headers.get(SSE_C_KEY_ID_HEADER)
+            key = EncryptionKey(secret=None, key_id=key_id)
+            return EncryptionSetting(mode, algorithm, key)
+        return EncryptionSetting(EncryptionMode.NONE)
 
 
 SSE_NONE = EncryptionSetting(mode=EncryptionMode.NONE,)
