@@ -12,13 +12,12 @@ from abc import ABCMeta, abstractmethod
 
 import logging
 import os
-from ..download_dest import DownloadDestLocalFile
 from .encryption_provider import AbstractSyncEncryptionSettingsProvider
 from ..bucket import Bucket
 
-from ..raw_api import SRC_LAST_MODIFIED_MILLIS
+from ..http_constants import SRC_LAST_MODIFIED_MILLIS
 from ..transfer.outbound.upload_source import UploadSourceLocalFile
-from .file import B2File
+from .path import B2SyncPath
 from .report import SyncFileReporter
 
 logger = logging.getLogger(__name__)
@@ -210,18 +209,18 @@ class B2HideAction(AbstractAction):
 class B2DownloadAction(AbstractAction):
     def __init__(
         self,
-        source_file: B2File,
+        source_path: B2SyncPath,
         b2_file_name: str,
         local_full_path: str,
         encryption_settings_provider: AbstractSyncEncryptionSettingsProvider,
     ):
         """
-        :param b2sdk.v1.B2File source_file: the file to be downloaded
+        :param b2sdk.v1.B2SyncPath source_path: the file to be downloaded
         :param str b2_file_name: b2_file_name
         :param str local_full_path: a local file path
         :param b2sdk.v1.AbstractSyncEncryptionSettingsProvider encryption_settings_provider: encryption setting provider
         """
-        self.source_file = source_file
+        self.source_path = source_path
         self.b2_file_name = b2_file_name
         self.local_full_path = local_full_path
         self.encryption_settings_provider = encryption_settings_provider
@@ -232,7 +231,7 @@ class B2DownloadAction(AbstractAction):
 
         :rtype: int
         """
-        return self.source_file.latest_version().size
+        return self.source_path.size
 
     def _ensure_directory_existence(self):
         parent_dir = os.path.dirname(self.local_full_path)
@@ -260,19 +259,18 @@ class B2DownloadAction(AbstractAction):
 
         # Download the file to a .tmp file
         download_path = self.local_full_path + '.b2.sync.tmp'
-        download_dest = DownloadDestLocalFile(download_path)
 
         encryption = self.encryption_settings_provider.get_setting_for_download(
             bucket=bucket,
-            file_version_info=self.source_file.latest_version().file_version_info,
+            file_version=self.source_path.selected_version,
         )
 
-        bucket.download_file_by_id(
-            self.source_file.latest_version().id_,
-            download_dest,
-            progress_listener,
+        downloaded_file = bucket.download_file_by_id(
+            self.source_path.selected_version.id_,
+            progress_listener=progress_listener,
             encryption=encryption,
         )
+        downloaded_file.save_to(download_path)
 
         # Move the file into place
         try:
@@ -289,13 +287,13 @@ class B2DownloadAction(AbstractAction):
         :type bucket: b2sdk.bucket.Bucket
         :param reporter: a place to report errors
         """
-        reporter.print_completion('dnload ' + self.source_file.name)
+        reporter.print_completion('dnload ' + self.source_path.relative_path)
 
     def __str__(self):
         return (
             'b2_download(%s, %s, %s, %d)' % (
-                self.b2_file_name, self.source_file.latest_version().id_, self.local_full_path,
-                self.source_file.latest_version().mod_time
+                self.b2_file_name, self.source_path.selected_version.id_, self.local_full_path,
+                self.source_path.mod_time
             )
         )
 
@@ -308,7 +306,7 @@ class B2CopyAction(AbstractAction):
     def __init__(
         self,
         b2_file_name: str,
-        source_file: B2File,
+        source_path: B2SyncPath,
         dest_b2_file_name,
         source_bucket: Bucket,
         destination_bucket: Bucket,
@@ -316,14 +314,14 @@ class B2CopyAction(AbstractAction):
     ):
         """
         :param str b2_file_name: a b2_file_name
-        :param b2sdk.v1.B2File source_file: the file to be copied
+        :param b2sdk.v1.B2SyncPath source_path: the file to be copied
         :param str dest_b2_file_name: a name of a destination remote file
         :param Bucket source_bucket: bucket to copy from
         :param Bucket destination_bucket: bucket to copy to
         :param b2sdk.v1.AbstractSyncEncryptionSettingsProvider encryption_settings_provider: encryption setting provider
         """
         self.b2_file_name = b2_file_name
-        self.source_file = source_file
+        self.source_path = source_path
         self.dest_b2_file_name = dest_b2_file_name
         self.encryption_settings_provider = encryption_settings_provider
         self.source_bucket = source_bucket
@@ -335,7 +333,7 @@ class B2CopyAction(AbstractAction):
 
         :rtype: int
         """
-        return self.source_file.latest_version().size
+        return self.source_path.size
 
     def do_action(self, bucket, reporter):
         """
@@ -352,24 +350,24 @@ class B2CopyAction(AbstractAction):
 
         source_encryption = self.encryption_settings_provider.get_source_setting_for_copy(
             bucket=self.source_bucket,
-            source_file_version_info=self.source_file.latest_version().file_version_info,
+            source_file_version=self.source_path.selected_version,
         )
 
         destination_encryption = self.encryption_settings_provider.get_destination_setting_for_copy(
             bucket=self.destination_bucket,
-            source_file_version_info=self.source_file.latest_version().file_version_info,
+            source_file_version=self.source_path.selected_version,
             dest_b2_file_name=self.dest_b2_file_name,
         )
 
         bucket.copy(
-            self.source_file.latest_version().id_,
+            self.source_path.selected_version.id_,
             self.dest_b2_file_name,
-            length=self.source_file.latest_version().size,
+            length=self.source_path.size,
             progress_listener=progress_listener,
             destination_encryption=destination_encryption,
             source_encryption=source_encryption,
-            source_file_info=self.source_file.latest_version().file_version_info.file_info,
-            source_content_type=self.source_file.latest_version().file_version_info.content_type,
+            source_file_info=self.source_path.selected_version.file_info,
+            source_content_type=self.source_path.selected_version.content_type,
         )
 
     def do_report(self, bucket, reporter):
@@ -380,13 +378,13 @@ class B2CopyAction(AbstractAction):
         :type bucket: b2sdk.bucket.Bucket
         :param reporter: a place to report errors
         """
-        reporter.print_completion('copy ' + self.source_file.name)
+        reporter.print_completion('copy ' + self.source_path.relative_path)
 
     def __str__(self):
         return (
             'b2_copy(%s, %s, %s, %d)' % (
-                self.b2_file_name, self.source_file.latest_version().id_, self.dest_b2_file_name,
-                self.source_file.latest_version().mod_time
+                self.b2_file_name, self.source_path.selected_version.id_, self.dest_b2_file_name,
+                self.source_path.mod_time
             )
         )
 
