@@ -13,11 +13,21 @@ from b2sdk.v1.account_info import (
     AbstractAccountInfo, InMemoryAccountInfo, UrlPoolAccountInfo, SqliteAccountInfo, StubAccountInfo
 )
 from b2sdk.v1.api import B2Api
+from b2sdk.v1.b2http import B2Http
 from b2sdk.v1.bucket import Bucket, BucketFactory
 from b2sdk.v1.cache import AbstractCache
+from b2sdk.v1.download_dest import (
+    AbstractDownloadDestination, DownloadDestLocalFile, PreSeekedDownloadDest, DownloadDestBytes,
+    DownloadDestProgressWrapper
+)
+from b2sdk.v1.exception import CommandError, DestFileNewer
+from b2sdk.v1.file_metadata import FileMetadata
 from b2sdk.v1.file_version import FileVersionInfo
 from b2sdk.v1.session import B2Session
 from b2sdk.v1.sync import (
     ScanPoliciesManager, DEFAULT_SCAN_MANAGER, zip_folders, Synchronizer, AbstractFolder,
-    LocalFolder, B2Folder, parse_sync_folder
+    LocalFolder, B2Folder, parse_sync_folder, SyncReport, File, B2File, FileVersion,
+    AbstractSyncEncryptionSettingsProvider
 )
+
+B2RawApi = B2RawHTTPApi
